@@ -1,7 +1,7 @@
 
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 Stas Zytkiewicz stas.zytkiewicz@gmail.com
+# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
 #
 #           packid.py
 # This program is free software; you can redistribute it and/or
@@ -21,32 +21,22 @@
 #create logger, logger was setup in SPLogging
 import logging
 # In your Activity class -> 
-# self.logger =  logging.getLogger("schoolsplay.packid.Activity")
+# self.logger =  logging.getLogger("childsplay.packid.Activity")
 # self.logger.error("I don't understand logger")
 # See SP manual for more info 
-  
-module_logger = logging.getLogger("schoolsplay.packid")
+
+module_logger = logging.getLogger("childsplay.packid")
 
 # standard modules you probably need
-import os,random,operator,string,sys
+import os,sys, random
 
 import pygame
 from pygame.constants import *
 
-from childsplay_sp.SPConstants import *
-import childsplay_sp.SPSpriteUtils as SPSpriteUtils
-from childsplay_sp.utils import load_image,load_sound,load_music,MyError,\
-                                font2surf,MazeGen,get_files,read_unicode_file,\
-                                ascii2hex, char2surf, speak_letter
-                                
-#0= wall 1=go
-#matrix =   ((0,0,0,0,0),\
-#            (0,1,0,1,0),\
-#            (0,1,0,1,0),\
-#            (0,1,1,1,0),\
-#            (0,0,0,0,0))
+import utils
+from SPConstants import *
+import SPSpriteUtils
 
-                                
 # containers that can be used globally to store stuff
 class Img:
     pass
@@ -139,19 +129,19 @@ class Memory:
 
 class Letters(Memory):
     instance = 0
-    def __init__(self,char,fcol,dest,ttf):
+    def __init__(self,char,fcol,dest,ttf, y):
         Letters.instance += 2
         self.wait = Letters.instance
         Memory.__init__(self)
 #        print "char",char
         self.char = char
-        self.image = char2surf(char,16,fcol,ttf, bold=True)
+        self.image = utils.char2surf(char,16,fcol,ttf, bold=True)
         self.rect = self.image.get_rect()
         self.lifetime = 50 # if we get in a loop
         # calculate the pix pos row*24+60, col*24+150
         self.row = 8
         self.col = 12
-        self.rect.move_ip(self.col*24+150,self.row*24+60)
+        self.rect.move_ip(self.col*24+150,self.row*24+60+y)
         self.dest = dest
         
         self.old_move = None
@@ -291,11 +281,11 @@ class Word:
 
     def _render(self):
         #print self.index
-        high_letter,lettersize = font2surf(self.chars[self.index],self.fontsize,self.hfcol,self.ttf,1, bold=True)
+        high_letter,lettersize = utils.text2surf(self.chars[self.index],self.fontsize,self.hfcol,self.ttf,1, bold=True)
 
         if self.index == 0: # Highlight first letter
             split = self.chars[1:]
-            word,wordsize = font2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
+            word,wordsize = utils.text2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
             surfsize = (lettersize[0]+wordsize[0],lettersize[1])
             surf = pygame.Surface(surfsize).convert()
             if self.cpg.get_localesetting()[1]: # RTL
@@ -307,7 +297,7 @@ class Word:
 
         elif self.index == len(self.chars)-1: # Highlight last letter
             split = self.chars[:-1]
-            word,wordsize = font2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
+            word,wordsize = utils.text2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
             surfsize = (lettersize[0]+wordsize[0],lettersize[1])
             surf = pygame.Surface(surfsize).convert()
             if self.cpg.get_localesetting()[1]: # RTL
@@ -318,9 +308,9 @@ class Word:
                 surf.blit(high_letter,(wordsize[0],0))
 
         else: # Letter inside the word, the word must be split in two parts
-            word,wordsize = font2surf(self.chars[:self.index],self.fontsize,\
+            word,wordsize = utils.text2surf(self.chars[:self.index],self.fontsize,\
                             self.fcol,self.ttf,1, bold=True)
-            word1,word1size = font2surf(self.chars[self.index+1:],self.fontsize,\
+            word1,word1size = utils.text2surf(self.chars[self.index+1:],self.fontsize,\
                               self.fcol,self.ttf,1, bold=True)
             surfsize = (lettersize[0]+wordsize[0]+word1size[0],lettersize[1])
             surf = pygame.Surface(surfsize).convert()
@@ -336,13 +326,14 @@ class Word:
         return self.surf
 
 class SidePanel:
-    def __init__(self,levels,ttf,cpg):
+    def __init__(self,levels,ttf,blitpos):
         self.ttf = ttf
         self.surf = pygame.Surface((130,400)).convert()
         pygame.draw.rect(self.surf,(42,191,44),self.surf.get_rect(),6)
         self.when = 9# number of gooditems to play last level
         self.level_pos_list = [(10,10),(10,110),(10,210),(10,310)]
         self.inlevel_word_pos_list = [(26,10),(26,110),(26,210),(26,310)]
+        
         self.x = 30
         self.y = 6
         self.fsize = 12
@@ -372,12 +363,12 @@ class SidePanel:
     
     def _set_level(self,levelstr,level):
         pos = self.level_pos_list[level-1]
-        levelimg = char2surf(levelstr,self.fsize+2,self.fcol,self.ttf, \
+        levelimg = utils.char2surf(levelstr,self.fsize+2,self.fcol,self.ttf, \
                              antialias=True, bold=True)
         self.surf.blit(levelimg,pos)
                              
     def set_word(self,word):
-        wordimg, size = font2surf(word,self.fsize-2,self.fcol,self.ttf,sizel=True,\
+        wordimg, size = utils.text2surf(word,self.fsize-2,self.fcol,self.ttf,sizel=True,\
                                    antialias=True, bold=True)
         self.y += size[1]
         self.surf.blit(wordimg,(self.x,self.y))
@@ -414,7 +405,7 @@ class LastLevelImg:
     def __init__(self,music,img,pos):
         self.char = 'FRUIT' # to be compatible with the letters class in the loop
         self.file = os.path.join(Img.datadir,music)
-        self.img = load_image(os.path.join(Img.datadir,img),1)
+        self.img = utils.load_image(os.path.join(Img.datadir,img),1)
         self.pos = pos
         self.rect = self.img.get_rect()
         # calculate the pix pos row*24+60, col*24+150
@@ -422,7 +413,7 @@ class LastLevelImg:
         self.rect.move_ip(self.col*24+150,self.row*24+60)
         
     def eat(self):
-        load_music(self.file).play()
+        utils.load_music(self.file).play()
         return 25 #score
 
 
@@ -430,45 +421,71 @@ class Activity:
     """  Base class mandatory for any SP activty.
     The activity is started by instancing this class by the core.
     This class must at least provide the following methods.
-    start (self) called by the core before calling loop.
+    start (self) called by the core before calling 'next_level'.
+    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
     next_level (self,level) called by the core when the user changes levels.
     loop (self,events) called 40 times a second by the core and should be your 
                       main eventloop.
-    helptitle (self) must return the title of the game can be localized.
-    help (self) must return a list of strings describing the activty.
-    name (self) must provide the activty name in english, not localized in lowercase.
+    get_helptitle (self) must return the title of the game can be localized.
+    get_help (self) must return a list of strings describing the activty.
+    get_helptip (self) must return a list of strings or an empty list
+    get_name (self) must provide the activty name in english, not localized in lowercase.
+    stop_timer (self) must stop any timers if any.
   """
 
     def __init__(self,SPGoodies):
         """SPGoodies is a class object that SP sets up and will contain references
         to objects, callback methods and observers
         TODO: add more explaination"""
-        self.logger =  logging.getLogger("schoolsplay.packid.Activity")
+        self.logger =  logging.getLogger("childsplay.packid.Activity")
         self.logger.info("Activity started")
         self.SPG = SPGoodies
+        self.scoredisplay = self.SPG.get_scoredisplay()
+        self.theme = self.SPG.get_theme()
         self.screen = self.SPG.get_screen()
+        self.screenclip = self.SPG.get_screenclip()
+        self.blit_pos = self.screenclip.left, self.screenclip.top
+        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
+        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
         self.backgr = self.SPG.get_background()
+        # The location of the activities Data dir
         self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','PackidData')
-        self.CPsounddir = os.path.join(self.SPG.get_libdir_path(),'CPData')
+        # Location of the CPData dir which holds some stuff used by multiple activities
+        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
+        # Location of the alphabet sounds dir
         self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
-        self.language = self.SPG.get_localesetting()[0]
-        self.ttf = None
+        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'c'))
+        self.rchash['theme'] = self.theme
+        self.logger.debug("found rc: %s" % self.rchash)
         # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
         # it returns a reference to the special CPGroup
         self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
+        
+        self.language = self.SPG.get_localesetting()[0]
+        self.ttf = None
         self.stopflag = None# used to stop the loop when not level 4
         self.score = 0
         self.letters_spots = []
         self.gamelevels = range(4)
         self.oldvolume = 0
-        
+    
     def __del__(self):
         #print 'Reached del'
         try:
             Snd.walk.stop()
         except:
             pass  
-            
+    
+    def clear_screen(self):
+        self.screen.blit(self.orgscreen,self.blit_pos)
+        self.backgr.blit(self.orgscreen,self.blit_pos)
+        pygame.display.update()
+
+    def refresh_sprites(self):
+        """Mandatory method, called by the core when the screen is used for blitting
+        and the possibility exists that your sprites are affected by it."""
+        self.actives.refresh()
+
     def get_helptitle(self):
         """Mandatory method"""
         return _("Packid")
@@ -487,7 +504,7 @@ class Activity:
         _("Try to find the way out while eating the fruits for extra points and"),
         _("funny sounds :-)")]
         return text 
-        
+    
     def get_helptip(self):
         """Mandatory method, when no tips available returns an empty list"""
         return []
@@ -498,34 +515,59 @@ class Activity:
         #                     Language, Alphabet, Fun, Miscellaneous
         return _("Fun/Alphabet")
         
+    
     def get_helplevels(self):
         """Mandatory method, must return a string with the number of levels
         in the follwing format:
-        _("This level has %s levels" % number-of-levels)"""
+        _("This level has %s levels") % number-of-levels"""
         return _("This activity has %s levels") % 4
     
+    def pre_level(self,level):
+        """Mandatory method.
+        Return True to call the eventloop after this method is called."""
+        self.logger.debug("pre_level called with: %s" % level)
+        pass
+        
+    def next_level(self,level,dbmapper):
+        """Mandatory method.
+        Return True if there levels left.
+        False when no more levels left."""
+        self.logger.debug("nextlevel called with: %s" % level)
+        if level == 5:
+            return False
+        self.level = level
+        self.dbmapper = dbmapper
+        # Your top blit position, this depends on the menubar position 
+        if self.blit_pos[1] == 0:
+            y = 10
+            self.gamearea = pygame.Rect(0, y, 800, 490) 
+        else:
+            y = 110
+            self.gamearea = pygame.Rect(0, y, 800, 600) 
+        return True
+    
     def start(self):
         """Mandatory method."""
-        Snd.waka = load_music(os.path.join(self.my_datadir,'waka.wav'))
-        Snd.walk = load_sound(os.path.join(self.my_datadir,'walk.wav'))
-        Snd.finlevel = load_sound(os.path.join(self.my_datadir,'finlevel.wav'))
-        Snd.bummer = load_sound(os.path.join(self.CPsounddir,'bummer.wav'))
-        Snd.eat = load_sound(os.path.join(self.my_datadir,'eat.wav'))
+        Snd.waka = utils.load_music(os.path.join(self.my_datadir,'waka.wav'))
+        Snd.walk = utils.load_sound(os.path.join(self.my_datadir,'walk.wav'))
+        Snd.finlevel = utils.load_sound(os.path.join(self.my_datadir,'finlevel.wav'))
+        Snd.bummer = utils.load_sound(os.path.join(self.CPdatadir,'bummer.wav'))
+        Snd.eat = utils.load_sound(os.path.join(self.my_datadir,'eat.wav'))
         
         Img.datadir = self.my_datadir
 
-        Img.pac_u = load_image(os.path.join(self.my_datadir,'pac_u.png'),1)
-        Img.pac_u_c = load_image(os.path.join(self.my_datadir,'pac_u_c.png'),1)
-        Img.pac_d = load_image(os.path.join(self.my_datadir,'pac_d.png'),1)
-        Img.pac_d_c = load_image(os.path.join(self.my_datadir,'pac_d_c.png'),1)
-        Img.pac_l = load_image(os.path.join(self.my_datadir,'pac_l.png'),1)
-        Img.pac_l_c = load_image(os.path.join(self.my_datadir,'pac_l_c.png'),1)
-        Img.pac_r = load_image(os.path.join(self.my_datadir,'pac_r.png'),1)
-        Img.pac_r_c = load_image(os.path.join(self.my_datadir,'pac_r_c.png'),1)
-        Img.pac_smile = load_image(os.path.join(self.my_datadir,'pac_smile.png'),0)
-        Img.pac_sad = load_image(os.path.join(self.my_datadir,'pac_sad.png'),0)
+        Img.pac_u = utils.load_image(os.path.join(self.my_datadir,'pac_u.png'),1)
+        Img.pac_u_c = utils.load_image(os.path.join(self.my_datadir,'pac_u_c.png'),1)
+        Img.pac_d = utils.load_image(os.path.join(self.my_datadir,'pac_d.png'),1)
+        Img.pac_d_c = utils.load_image(os.path.join(self.my_datadir,'pac_d_c.png'),1)
+        Img.pac_l = utils.load_image(os.path.join(self.my_datadir,'pac_l.png'),1)
+        Img.pac_l_c = utils.load_image(os.path.join(self.my_datadir,'pac_l_c.png'),1)
+        Img.pac_r = utils.load_image(os.path.join(self.my_datadir,'pac_r.png'),1)
+        Img.pac_r_c = utils.load_image(os.path.join(self.my_datadir,'pac_r_c.png'),1)
+        Img.pac_smile = utils.load_image(os.path.join(self.my_datadir,'pac_smile.png'),0)
+        Img.pac_sad = utils.load_image(os.path.join(self.my_datadir,'pac_sad.png'),0)
         
-        self.sidepan = SidePanel(self.gamelevels,self.ttf,self.SPG)
+        self.sidepan = SidePanel(self.gamelevels,self.ttf,self.blit_pos)
         
         # test if we have a words list for the locale
         try:
@@ -538,24 +580,24 @@ class Activity:
             wordlist = 'words-en'
             wordsloc = 'en'
         try:
-            items = read_unicode_file(os.path.join(self.my_datadir,wordlist))
+            items = utils.read_unicode_file(os.path.join(self.my_datadir,wordlist))
         except IOError,info:
             self.logger.exception("can't open or read words file %s" % \
                                   os.path.join(self.my_datadir,wordlist))
             raise MyError
         if self.SPG.get_localesetting()[1]:
-            self.words = filter(None,map(operator.getslice,items,\
-                                        (0,)*len(items),(-1,)*len(items)))
+            self.words = [s[:-1] for s in items]
         else:
-            self.words = map(string.upper,filter(None,map(operator.getslice,items,\
-                                        (0,)*len(items),(-1,)*len(items))))
-        
+            self.words = [s[:-1].upper() for s in items]
+        #print self.words
         # Look for alphabet sounds for this locale.
         # If all fails we fall back to the "old" wahoo sound.
         self.alphabetdir = self.SPG.get_absdir_loc_path()
         if self.alphabetdir == 'en' and self.alphabetdir not in self.language:
             # if true the sound dir wasn't found and 'en' is returnt as fallback
             self.alphabetdir = None
+        self.SPG.tellcore_set_dice_minimal_level(3)
+        
         
     def _setup_last_level(self):
         self.fruits,spots = [],[]
@@ -571,42 +613,94 @@ class Activity:
             self.fruits.append((LastLevelImg(music.pop(),file,spot)))
         self.letters_spots.append(((15,23)))    
     
-    def next_level(self,level,dbmapper):
-        """Mandatory method.
-        Return True if there levels left.
-        False when no more levels left."""
-        if level == 5:
-            return False
-        self.level = level
-        self.dbmapper = dbmapper
-        return True
-    
     def post_next_level(self):
         """Mandatory method.
-        This is called by the core after 'next_level' *and* after the 321 count.
+        This is called once by the core after 'next_level' *and* after the 321 count.
         You should place stuff in here that run in a seperate thread like sound play."""
         self.next_exercise()
+        
+    def get_score(self,timespend):
+        """Mandatory method.
+        @timespend is the time spend inside the level as it's calculated
+        by the core.
+        returns the score value for the past level.
+        return None if no score value is used"""
+        # As this is called when a level is finished we also use it to store
+        # dbase stuff.
+        m,s = timespend.split(':')
+        seconds = int(m)*60 + int(s)
+        
+        self.dbmapper.insert('word',self.word)
+        self.dbmapper.insert('wrong',self.gw)
+        
+        score = max(1,(10-self.gw) - (seconds/150.0))
+        self.logger.debug("score is: %s" % score)
+        return score
+    
+    def stop_timer(self):
+        """You *must* provide a method to stop timers if you use them.
+        The SPC will call this just in case and catch the exception in case 
+        there's no 'stop_timer' method""" 
+        try:
+            self.timer.stop()
+        except:
+            pass
 
-    def next_exercise(self):    
-        pygame.display.update(self.screen.fill((0,0,0)))
+    def loop(self,events):
+        """Mandatory method.
+        This is the main eventloop called by the core 30 times a minute."""
+        if self.stopflag:
+            self.SPG.tellcore_game_end(True)
+        self.stop = 0
+        self.score = 0
+        for event in events:
+            pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
+            if event.type is KEYDOWN and not self.objs_to_move:
+                key = self._on_keypress(event.key)
+                if key:
+                    r = self.screen.blit(Img.backgr,self.packid.rect.inflate(2,2),self.packid.rect.inflate(2,2))
+                    self.pack_pos = self.packid.update(key)
+                    rr = self.screen.blit(self.packid.img,self.packid.rect)
+                    pygame.display.update((r,rr))
+        if self.objs_to_move: # Are there letters to move?
+            self._move_objs()
+        
+        elif self.pack_pos in self.letters_spots: # Have we hit a letter?
+            if self.level == 4 and self.pack_pos == (15,23):# pac hit exit in last level
+                self.SPG.tellcore_game_end(True)
+            else:
+                for item in self.letter_objs:
+                    if self.pack_pos == (item.row,item.col):
+                        if item.char == 'FRUIT':
+                            self.score = item.eat()
+                            self.letter_objs.remove(item)
+                            pygame.display.update(self.screen.blit(Img.backgr_start,item.rect,item.rect))# erase fruit
+                            self.packid.img = Img.pac_smile
+                            pygame.display.update(self.screen.blit(Img.pac_smile,self.packid.rect))
+                        else:
+                            self._check_letter(item)
+                        break
+        return 
+    def next_exercise(self): 
+        pygame.display.update(self.screen.fill((0,0,0), self.gamearea))
         self.letters_spots = []
         if self.level == 1:
-            self.brick = load_image(os.path.join(self.my_datadir,'brick.png'),0)
+            self.brick = utils.load_image(os.path.join(self.my_datadir,'brick.png'),0)
             matrix = self._read_grid('grid0.txt')
             self._build_field(matrix)
         elif self.level == 2:
-            self.brick = load_image(os.path.join(self.my_datadir,'leafs.png'),0)
+            self.brick = utils.load_image(os.path.join(self.my_datadir,'leafs.png'),0)
             matrix = self._read_grid('grid1.txt')
             self._build_field(matrix,1)
         elif self.level == 3:
-            self.brick = load_image(os.path.join(self.my_datadir,'sea.png'),0)
+            self.brick = utils.load_image(os.path.join(self.my_datadir,'sea.png'),0)
             matrix = self._read_grid('grid2.txt')
             self._build_field(matrix,1)
         elif self.level == 4:
             if self.sidepan.get_level_score():
                 load_music(os.path.join(self.my_datadir,'feelgood.ogg')).play()
                 
-                self.brick = load_image(os.path.join(self.my_datadir,'camo.png'),0)
+                self.brick = utils.load_image(os.path.join(self.my_datadir,'camo.png'),0)
                 m = MazeGen(17,25)
                 matrix = m.get_maze()
                 
@@ -630,16 +724,16 @@ class Activity:
         ### TODO fix RTL issues here
         if self.level != 4:
             if self.SPG.get_localesetting()[1]:
-                surf,spam = font2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
-                pygame.display.update(self.screen.blit(surf,(780-spam[0],0)))
-                surf,self.surfword_offset = font2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
-                pygame.display.update(self.screen.blit(surf,(780-self.surfword_offset[0],24)))
+                surf,spam = utils.text2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
+                pygame.display.update(self.screen.blit(surf,(780-spam[0],0+self.gamearea.top)))
+                surf,self.surfword_offset = utils.text2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
+                pygame.display.update(self.screen.blit(surf,(780-self.surfword_offset[0],24+self.gamearea.top)))
                 word = random.choice(self.words)
             else:
-                surf,spam = font2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
-                pygame.display.update(self.screen.blit(surf,(28,4)))
-                surf,self.surfword_offset = font2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
-                pygame.display.update(self.screen.blit(surf,(8,30)))
+                surf,spam = utils.text2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
+                pygame.display.update(self.screen.blit(surf,(28,4+self.gamearea.top)))
+                surf,self.surfword_offset = utils.text2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
+                pygame.display.update(self.screen.blit(surf,(8,30+self.gamearea.top)))
                 word = random.choice(self.words)
         
         # start the show and set some initial values
@@ -660,7 +754,7 @@ class Activity:
             self.surfword = Word(word,self.ttf,self.SPG)
             self._update_surfword()
         x = 174
-        y = 84
+        y = 84 + self.gamearea.top
         self.packid = PacKid(matrix,x,y,24)
         
         self.pack_pos = (self.packid.row,self.packid.col)# start postion
@@ -681,16 +775,16 @@ class Activity:
 #                    (780-self.surfword_offset[0]-65,30),cutoff_rect))
 #        else:
         pygame.display.update(self.screen.blit(surf,\
-                        (8+self.surfword_offset[0],30)))
+                        (8+self.surfword_offset[0],30+self.gamearea.top)))
         pygame.time.wait(1000)
     
     def _speak_letter(self,letter):
         """Plays the alphabet soundfile for @letter
         Return True on succes and False on failure"""
-        return speak_letter(letter.lower(), self.language)
+        return utils.speak_letter(letter.lower(), self.language)
            
     def _update_sidepanel(self):
-        pygame.display.update(self.screen.blit(self.sidepan.get_surf(),(0,60)))
+        pygame.display.update(self.screen.blit(self.sidepan.get_surf(),(0,60 + self.gamearea.top)))
     
     def _read_grid(self,name):
         grid = []
@@ -701,7 +795,7 @@ class Activity:
         f.close()
         return tuple(grid)
     
-    def _build_field(self,matrix,ran=0):
+    def _build_field(self,matrix, ran=0):
         Img.matrix = matrix #stash ref
         surf = pygame.Surface((620,428))
         y = 10
@@ -710,13 +804,13 @@ class Activity:
             for item in row:
                 if not item:
                     if ran:
-                        brick = self.brick.convert()
+                        brick = self.brick#.convert()
                         angle = random.choice((0,90,180,270))
                         surf.blit(pygame.transform.rotate(brick,angle),(x,y))
                     else:
                         surf.blit(self.brick,(x,y))
                 elif item == 2:# special
-                    i = load_image(os.path.join(self.my_datadir,'exit.png'),0)
+                    i = utils.load_image(os.path.join(self.my_datadir,'exit.png'),0)
                     #image = 48x48
                     #x -= 24
                     #y -= 24
@@ -724,7 +818,7 @@ class Activity:
                     break
                 x +=24
             y += 24
-        pygame.display.update(self.screen.blit(surf,(140,50)))
+        pygame.display.update(self.screen.blit(surf,(140,50 + self.gamearea.top)))
         
     def _start_letters(self,word):
         self.word,spots = word,[]
@@ -736,7 +830,7 @@ class Activity:
             if spot in spots or spot in ((8,11),(8,12),(8,13),(1,1)):# same coords or inside the box,packid
                 spot = self._rand_spot()
             spots.append((spot))
-            self.objs_to_move.append((Letters(item,(255,255,255),spot,self.ttf)))
+            self.objs_to_move.append((Letters(item,(255,255,255),spot,self.ttf, self.gamearea.top)))
             
             #print item,spot,Img.matrix[spot[0]][spot[1]]
 
@@ -782,7 +876,7 @@ class Activity:
                 if self.objs_to_move == []:
                     Snd.walk.stop()
                     Snd.walk.set_volume(self.oldvolume)
-                    r= self.screen.blit(self.packid.img,(174,84))# blit packid
+                    r= self.screen.blit(self.packid.img,self.packid.rect)# blit packid
                     pygame.display.update(r)
                     pygame.time.wait(500)
     
@@ -803,7 +897,7 @@ class Activity:
             
             self.score = 10
             self.letters_spots.remove(self.pack_pos)
-            print self.letters_spots
+            #print self.letters_spots
             if self.letters_spots:
                 self.letter_to_find += 1 
                 self._update_surfword()
@@ -828,57 +922,3 @@ class Activity:
             pygame.display.update(self.screen.blit(Img.pac_sad,self.packid.rect))
             pygame.time.wait(500)
         
-    def get_score(self,timespend):
-        """Mandatory method.
-        @timespend is the time spend inside the level as it's calculated
-        by the core.
-        returns the score value for the past level.
-        return None if no score value is used"""
-        # As this is called when a level is finished we also use it to store
-        # dbase stuff.
-        m,s = timespend.split(':')
-        seconds = int(m)*60 + int(s)
-        
-        self.dbmapper.insert('word',self.word)
-        self.dbmapper.insert('wrong',self.gw)
-        
-        score = max(1,(10-self.gw) - (seconds/150.0))
-        self.logger.debug("score is: %s" % score)
-        return score
-        
-    def loop(self,events):
-        """Mandatory method.
-        This is the main eventloop called by the core 30 times a minute."""
-        if self.stopflag:
-            self.SPG.tellcore_game_end(True)
-        self.stop = 0
-        self.score = 0
-        for event in events:
-            pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
-            if event.type is KEYDOWN and not self.objs_to_move:
-                key = self._on_keypress(event.key)
-                if key:
-                    r = self.screen.blit(Img.backgr,self.packid.rect.inflate(2,2),self.packid.rect.inflate(2,2))
-                    self.pack_pos = self.packid.update(key)
-                    rr = self.screen.blit(self.packid.img,self.packid.rect)
-                    pygame.display.update((r,rr))
-        if self.objs_to_move: # Are there letters to move?
-            self._move_objs()
-        
-        elif self.pack_pos in self.letters_spots: # Have we hit a letter?
-            if self.level == 4 and self.pack_pos == (15,23):# pac hit exit in last level
-                self.SPG.tellcore_game_end(True)
-            else:
-                for item in self.letter_objs:
-                    if self.pack_pos == (item.row,item.col):
-                        if item.char == 'FRUIT':
-                            self.score = item.eat()
-                            self.letter_objs.remove(item)
-                            pygame.display.update(self.screen.blit(Img.backgr_start,item.rect,item.rect))# erase fruit
-                            self.packid.img = Img.pac_smile
-                            pygame.display.update(self.screen.blit(Img.pac_smile,self.packid.rect))
-                        else:
-                            self._check_letter(item)
-                        break
-        return 
-        
